import gtk
import cairo
import math

class Image (object):

	def __init__(self, filename):
		self.surface = cairo.ImageSurface.create_from_png(filename)

	def draw(self, cr):
		cr.save()
		cr.set_source_surface(self.surface)
		cr.paint()
		cr.restore()


class Widget (gtk.DrawingArea):

	def __init__(self):
		gtk.DrawingArea.__init__(self)
		self.set_size_request(300, 300)
		self.connect('expose-event', self.on_expose)

	def refresh(self):
		rect = gtk.gdk.Rectangle(0, 0, self.allocation.width, self.allocation.height)
		self.window.invalidate_rect(rect, True)
		self.window.process_updates(True)

	def on_expose(self, widget, data=None):
		cr = self.window.cairo_create()
		cr.set_source_rgb(0, 0, 0)
		cr.paint()
		self.draw(cr)

	def draw(self, cr):
		pass


class Compass (Widget):

	def __init__(self):
		Widget.__init__(self)
		self.background = Image('compass-background.png')
		self.face = Image('compass-face.png')
		self.glass = Image('compass-glass.png')
		self.angle = 0.0

	def set_azimuth(self, azimuth):
		self.angle = -math.radians(azimuth)
		self.refresh()

	def draw(self, cr):
		cr.save()
		self.background.draw(cr)
		cr.save()
		cr.translate(150, 150)
		cr.rotate(self.angle)
		cr.translate(-150, -150)
		self.face.draw(cr)
		cr.restore()
		self.glass.draw(cr)
		cr.restore()


class Horizon (Widget):

	def __init__(self):
		Widget.__init__(self)
		self.face = Image('horizon-face.png')
		self.background = Image('horizon-background.png')
		self.hand_shadow = Image('horizon-hand-shadow.png')
		self.hand = Image('horizon-hand.png')
		self.glass = Image('horizon-glass.png')
		self.pitch = 0.0
		self.roll = 0.0

	def set_horizon(self, pitch, roll):
		self.pitch = pitch * 4
		self.roll = math.radians(roll)
		self.refresh()

	def draw_face(self, cr):
		cr.save()
		cr.arc(150, 150, 150, 0, 2*math.pi)
		cr.clip()
		cr.translate(0, -742-self.pitch)
		cr.translate(150, 892+self.pitch)
		cr.rotate(self.roll)
		cr.translate(-150, -892-self.pitch)
		self.face.draw(cr)
		cr.restore()

	def draw_hand(self, cr):
		cr.save()
		cr.translate(150, 150)
		cr.rotate(self.roll)
		cr.translate(-150, -150)
		self.hand.draw(cr)
		cr.restore()

	def draw_hand_shadow(self, cr):
		cr.save()
		cr.translate(3, 3)
		cr.translate(150, 150)
		cr.rotate(self.roll)
		cr.translate(-150, -150)
		self.hand_shadow.draw(cr)
		cr.restore()

	def draw(self, cr):
		cr.save()
		self.draw_face(cr)
		self.background.draw(cr)
		self.draw_hand_shadow(cr)
		self.draw_hand(cr)
		self.glass.draw(cr)
		cr.restore()

class GForce (Widget):

	def __init__(self):
		Widget.__init__(self)
		self.face = Image('gforce-face.png')
		self.progress = [0, 0, 0, 0, 0, 0]

	def set_accel(self, x, y, z):
		if x > 0:
			self.progress[0] = x * 48.5
			self.progress[1] = 0
		else:
			self.progress[0] = 0
			self.progress[1] = -x * 48.5
		
		if y > 0:
			self.progress[2] = y * 48.5
			self.progress[3] = 0
		else:
			self.progress[2] = 0
			self.progress[3] = -y * 48.5
		
		if z > 0:
			self.progress[4] = z * 48.5
			self.progress[5] = 0
		else:
			self.progress[4] = 0
			self.progress[5] = -z * 48.5
		
		self.refresh()

	def draw(self, cr):
		cr.save()
		self.face.draw(cr)

		cr.set_source_rgb(0.937, 0.161, 0.161)
		cr.rectangle(130, 125, 40, -self.progress[2])
		cr.fill()

		cr.set_source_rgb(0.451, 0.824, 0.086)
		cr.rectangle(130, 175, 40, self.progress[3])
		cr.fill()

		cr.set_source_rgb(0.929, 0.831, 0.000)
		cr.rectangle(125, 130, -self.progress[1], 40)
		cr.fill()

		cr.rectangle(175, 130, self.progress[0], 40)
		cr.fill()

		cr.set_source_rgb(0.447, 0.624, 0.812)
		cr.rectangle(232, 125, 40, -self.progress[5])
		cr.fill()

		cr.rectangle(232, 175, 40, self.progress[4])
		cr.fill()
		cr.restore()


class Level (Widget):

	def __init__(self):
		Widget.__init__(self)
		self.level = 0
		self.background = Image('level-background.png')
		self.hand = Image('level-hand.png')
		self.glass = Image('level-glass.png')

	def set_angle(self, angle):
		self.level = angle * 8
		self.refresh()

	def draw(self, cr):
		cr.save()
		cr.translate(22, 118)
		cr.arc(32, 32, 32, math.pi*0.5, math.pi*1.5)
		cr.rel_line_to(192, 0)
		cr.arc(224, 32, 32, math.pi*1.5, math.pi*0.5)
		cr.close_path()
		cr.clip()
		self.background.draw(cr)
		cr.save()
		cr.translate(86-self.level, 2)
		self.hand.draw(cr)
		cr.restore()
		self.glass.draw(cr)
		cr.restore()

